package com.holmos.cache;

import java.util.List;

import com.holmos.cache.data.HolmosCacheData;
import com.holmos.cache.data.HolmosDataResult;
import com.holmos.cache.data.HolmosResultCode;
import com.holmos.cache.state.HolmosCacheState;


/**
 * Wasten 平台用到的缓存系统的缓存接口
 * 
 * <li>用户可以对此接口进行扩展实现自己的缓存系统</li>
 * <li>Wasten平台的缓存系统，默认实现了内存的缓存和磁盘缓存两级缓存，用户可以根据自己的需求进行扩展</li>
 * 
 * @author 吴银龙(sunshine307087558@gmail.com)
 * */
public interface HolmosCache {
	
	/**
	 * 获得此缓存的级别，级别为1也就是说最先在这个地方访问数据
	 * <li>系统用到的所有缓存的级别不能相同，这个在进行加载配置文件的时候会进行校验,如果相同，则抛出异常</li>
	 * <li>级别设置从1开始，如果出现0或者负数，则抛出异常</li>
	 * <li>该字段为必须设置的字段</li>
	 * 
	 * @return 该缓存的缓存级别，based 1
	 * */
	public byte getLevel();
	
	/**
	 * 获取该缓存的名字
	 * <li>系统用到的所有缓存的名字不能相同，这个在进行加载配置文件的时候会进行校验，如果相同，抛出异常</li>
	 * <li>如果发现配置文件中没有设置或者设置为""，或者为null，则默认的名字为  "level[index]"</li>
	 * 
	 * @return 缓存的名字
	 * */
	public String getCacheName();
	
	/**
	 * 将元素放入缓存中
	 * 
	 * <li>data不能为null,如果为null，没有任何操作产生</li>
	 * <li>如果缓存中已有此元素，则为更新操作</li>
	 * 
	 * @param data 待存入缓存中的元素
	 * */
	public HolmosResultCode putData(HolmosCacheData data);
	
	/**
	 * 将数据批量放入缓存
	 * 
	 * <li>data不能为null,如果为null，没有任何操作产生</li>
	 * <li>如果缓存中已有此元素，则为更新操作</li>
	 * 
	 * @param data 待批量存入缓存中的元素
	 * */
	public HolmosResultCode putBatchData(List<HolmosCacheData> data);
	
	/**
	 * 从缓存中将索引为key的元素删除掉
	 * 
	 * <li>不做真实删除，只是将其标记改变，当缓存空间不够的时候，如有新元素过来，再将其删除</li>
	 * <li>如果key指示的元素不存在，则不进行任何操作</li>
	 * 
	 * @param key 指示待删除元素的索引
	 * */
	public HolmosResultCode removeData(Object key);
	
	/**
	 * 从缓存中批量删除与索引列表keys对应的数据
	 * 
	 * <li>不做真实删除，只是将其标记改变，当缓存空间不够的时候，如有新元素过来，再将其删除</li>
	 * <li>如果key指示的元素不存在，则不进行任何操作</li>
	 * 
	 * @param keys 指示待删除元素的索引列表
	 * */
	public HolmosResultCode removeBatchData(List<? extends Object> keys);
	/**
	 * 从缓存中获取索引为key的元素
	 * 
	 * <li>如果索引为key的元素不存在，则返回null</li>
	 * 
	 * @param key 获取缓存元素的索引
	 * @return 获取到的缓存元素
	 * */
	public HolmosDataResult<HolmosCacheData> getData(Object key);
	
	/**
	 * 批量获取数据，返回的数据的顺序与keys中key的顺序一致
	 * 
	 * <li>如果索引为key的元素不存在，则返回结果中对应的项为null</li>
	 * 
	 * @param keys 批量获取元素的索引列表
	 * @return 返回批量获取的数据
	 * */
	public HolmosDataResult<List<HolmosCacheData>> getBatchData(List<? extends Object> keys);
	
	/**
	 * 锁住一个key，不再允许更新, 允许读和删除。
	 * 
	 * @param key  数据的索引
	 * */
	public HolmosResultCode lock(Object key);
	
	/**
	 * 解锁一个key。
	 * 
	 * @param key  数据的key
	 * @return 如果数据不存在，返回不存在；如果数据存在但未被lock，返回lock不存在的错误码；
	 *         否则成功。
	 */
	public HolmosResultCode unlock(Object key);
	
	/**
	 * 批量锁key
	 * 
	 * @param keys  批量数据的索引
	 * @return 如果失败，返回失败操作码，否则成功
	 */
	public HolmosResultCode mlock(List<? extends Object> keys);

	/**
	 * 批量解锁key。
	 *
	 * @param keys  数据的key
	 * @return 如果失败，返回失败操作码，否则成功
	 */
	public HolmosResultCode munlock(List<? extends Object> keys);

	/**
	 * 清除缓存,quick用来代表是否是快速清除，如果是快速清除，那么直接清除缓存，并且回收内存资源，否则，在实际清除之前，将缓存中内容写入持久层
	 * 
	 * @param quick 用来指示是否是快速清除
	 * */
	public HolmosResultCode clear(boolean quick);
	
	/**
	 * 获得当前缓存的状态信息
	 * 
	 * @return 当前缓存的状态信息
	 * */
	public HolmosCacheState getCacheState();
	
	/**
	 * 删除过期的缓存元素，quick 用来指示是否快速删除，如果是快速删除，则直接从缓存中删除，<br>
	 * 并回收缓存资源；如果不是则先将其写入持久层，再回收缓存资源
	 * */
	public HolmosResultCode removeExpireData(boolean quick);
	
	/**
	 * 移除处于删除状态的缓存元素
	 * 
	 * */
	public HolmosResultCode removeDeletedData();
	
	/**
	 * 增加集合数据类型
	 * 
	 * <li>如果原始数据不是集合类型，不做任何操作，直接返回错误码</li>
	 * <li>如果索引key指定的元素不存在，则执行insert操作</li>
	 * 
	 * @param key 数据的索引
	 * @param items 要增加的value
	 * @param offset 添加数据的起始偏移量
	 * @param maxCount 集合允许的最大条目数量，超过这个数量，系统将直接删除相应数量的最早放入的条目
	 * @return 返回代码
	 */
	public HolmosResultCode addItems(Object key,List<? extends Object> items, 
			int offset,int maxCount);

	/**
	 * 删除集合中的数据
	 * 
	 * <li>如果原始数据不是集合类型，不做任何操作，直接返回错误码</li>
	 * <li>如果索引key指定的元素不存在，不做任何操作，直接返回错误码</li>
	 * 
	 * @param key 数据的索引
	 * @param offset 要删除的数据的偏移量
	 * @param count 要删除的数据的条数
	 * @return 删除是否成功
	 */
	public HolmosResultCode removeItems(Object key, int offset,
			int count);

	/**
	 * 删除并返回集合中的数据
	 * 
	 * <li>如果原始数据不是集合类型，不做任何操作，直接返回错误码</li>
	 * <li>如果索引key指定的元素不存在，不做任何操作，直接返回</li>
	 * 
	 * @param key 数据的key
	 * @param offset 要删除的数据的偏移量
	 * @param count 要删除的数据的条数
	 * 
	 * @return 如果删除成功，返回本次删除成功删除的数据
	 */
	public HolmosDataResult<HolmosCacheData> getAndRemove(Object key, int offset, int count);

}
