package com.holmos.cache.store;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import com.holmos.cache.data.HolmosCacheData;
import com.holmos.cache.data.HolmosDataResult;
import com.holmos.cache.data.HolmosResultCode;
import com.holmos.cache.state.HolmosCacheState;

/**
 * Watsen 平台缓存数据仓库的接口
 * 
 * @author 吴银龙(sunshine307087558@gmail.com)
 * 
 * */
public interface HolmosCacheStore {
	/**
	 * 将元素放入缓存中
	 * 
	 * <li>data不能为null,如果为null，没有任何操作产生</li>
	 * <li>如果缓存中已有此元素，则为更新操作</li>
	 * 
	 * @param data 待存入缓存中的元素
	 * */
	public HolmosResultCode putData(HolmosCacheData data);
	
	/**
	 * 将数据批量放入缓存
	 * 
	 * <li>data不能为null,如果为null，没有任何操作产生</li>
	 * <li>如果缓存中已有此元素，则为更新操作</li>
	 * 
	 * @param data 待批量存入缓存中的元素
	 * */
	public HolmosResultCode putBatchData(List<HolmosCacheData> data);
	
	/**
	 * 从缓存中将索引为key的元素删除掉
	 * 
	 * <li>如果key指示的元素不存在，则不进行任何操作</li>
	 * 
	 * @param key 指示待删除元素的索引
	 * */
	public HolmosResultCode removeData(Object key);
	
	/**
	 * 从缓存中批量删除与索引列表keys对应的数据
	 * 
	 * <li>不做真实删除，只是将其标记改变，当缓存空间不够的时候，如有新元素过来，再将其删除</li>
	 * <li>如果key指示的元素不存在，则不进行任何操作</li>
	 * 
	 * @param keys 指示待删除元素的索引列表
	 * */
	public HolmosResultCode removeBatchData(List<? extends Object> keys);
	/**
	 * 从缓存中获取索引为key的元素
	 * 
	 * <li>如果索引为key的元素不存在，则返回null</li>
	 * 
	 * @param key 获取缓存元素的索引
	 * @return 获取到的缓存元素
	 * */
	public HolmosDataResult<HolmosCacheData> getData(Object key);
	
	/**
	 * 批量获取数据，返回的数据的顺序与keys中key的顺序一致
	 * 
	 * <li>如果索引为key的元素不存在，则返回结果中对应的项为null</li>
	 * 
	 * @param keys 批量获取元素的索引列表
	 * @return 返回批量获取的数据
	 * */
	public HolmosDataResult<List<HolmosCacheData>> getBatchData(List<? extends Object> keys);
	
	/**
	 * 锁住一个key，不再允许更新, 允许读和删除。
	 * 
	 * @param key  数据的索引
	 * */
	public HolmosResultCode lock(Object key);
	
	/**
	 * 判断索引key所指的元素是否被锁
	 * 
	 * @param key 数据的索引
	 * @return true 被锁住<br>false 没被锁
	 * */
	public boolean isLocked(Object key);
	
	/**
	 * 解锁一个key。
	 * 
	 * @param key  数据的key
	 * @return 如果数据不存在，返回不存在；如果数据存在但未被lock，返回lock不存在的错误码；
	 *         否则成功。
	 */
	public HolmosResultCode unlock(Object key);
	
	/**
	 * 批量锁key
	 * 
	 * @param keys  批量数据的索引
	 * @return 如果失败，返回失败操作码，否则成功
	 */
	public HolmosResultCode mlock(List<? extends Object> keys);

	/**
	 * 批量解锁key。
	 *
	 * @param keys  数据的key
	 * @return 如果失败，返回失败操作码，否则成功
	 */
	public HolmosResultCode munlock(List<? extends Object> keys);

	/**
	 * 清除缓存,quick用来代表是否是快速清除，如果是快速清除，那么直接清除缓存，并且回收内存资源，否则，在实际清除之前，将缓存中内容写入持久层
	 * 
	 * @param quick 用来指示是否是快速清除
	 * */
	public HolmosResultCode clear(boolean quick);
	
	/**
	 * 删除过期的缓存元素，quick 用来指示是否快速删除，如果是快速删除，则直接从缓存中删除，<br>
	 * 并回收缓存资源；如果不是则先将其写入持久层，再回收缓存资源
	 * */
	public HolmosResultCode removeExpireData(boolean quick);
	
	/**
	 * 移除处于删除状态的缓存元素
	 * 
	 * */
	public HolmosResultCode removeDeletedData();
	
	/**
	 * 增加集合数据类型
	 * 
	 * <li>如果原始数据不是集合类型，不做任何操作，直接返回错误码</li>
	 * <li>如果索引key指定的元素不存在，则执行insert操作</li>
	 * 
	 * @param key 数据的索引
	 * @param items 要增加的value
	 * @param offset 添加数据的起始偏移量
	 * @param maxCount 集合允许的最大条目数量，超过这个数量，系统将直接删除相应数量的最早放入的条目
	 * @return 返回代码
	 */
	public HolmosResultCode addItems(Object key,List<? extends Object> items, 
			int offset,int maxCount);

	/**
	 * 删除集合中的数据
	 * 
	 * <li>如果原始数据不是集合类型，不做任何操作，直接返回错误码</li>
	 * <li>如果索引key指定的元素不存在，不做任何操作，直接返回错误码</li>
	 * 
	 * @param key 数据的索引
	 * @param offset 要删除的数据的偏移量
	 * @param count 要删除的数据的条数
	 * @return 删除是否成功
	 */
	public HolmosResultCode removeItems(Object key, int offset,
			int count);

	/**
	 * 删除并返回集合中的数据
	 * 
	 * <li>如果原始数据不是集合类型，不做任何操作，直接返回错误码</li>
	 * <li>如果索引key指定的元素不存在，不做任何操作，直接返回</li>
	 * 
	 * @param key 数据的key
	 * @param offset 要删除的数据的偏移量
	 * @param count 要删除的数据的条数
	 * 
	 * @return 如果删除成功，返回本次删除成功删除的数据
	 */
	public HolmosDataResult<HolmosCacheData> getAndRemove(Object key, int offset, int count);
	
	/**
	 * 获得数据仓库当前的信息
	 * 
	 * @return 数据仓库当前的信息
	 * */
	public HolmosCacheState getCacheState();
	
	/**
	 * 获得缓存元素的数量
	 * 
	 * @return 缓存的大小
	 * */
	public long getStoreElementsCount();
	
	/**
	 * 获得当前缓存的大小，以字节为单位
	 * 
	 * @return 缓存大小，以字节为单位
	 * */
	public long getStoreSize();
	
	public Set<Object> getAllKeys();
	
	public ArrayList<HolmosCacheData> getAllDatas();
}
