package com.holmos.cache.store;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import com.holmos.cache.constvalue.CacheConstValue;
import com.holmos.cache.data.HolmosCacheData;
import com.holmos.cache.data.HolmosDataResult;
import com.holmos.cache.data.HolmosResultCode;
import com.holmos.cache.size.layout.sizegetter.ReflectSizeGetter;
import com.holmos.cache.size.layout.sizegetter.SizeGetter;
import com.holmos.cache.size.layout.sizegetter.SizeGetterFactory;
import com.holmos.cache.state.HolmosCacheState;

/**
 * 内存类型缓存，采用ConcurrentHashMap高速并发HashTable进行存储
 * 
 * @author 吴银龙(sunshine307087558@gmail.com)
 * */
public class HolmosMemoryCacheStore implements HolmosCacheStore{
	/**缓存大小*/
	private long cacheSize;
	
	/**缓存当前状态*/
	private HolmosCacheState cacheState=new HolmosCacheState();
	
	
	
	private static ReflectSizeGetter sizeGetter=null;
	static{
		ArrayList<SizeGetter> sizeGetters=new ArrayList<SizeGetter>();
		sizeGetters.add(SizeGetterFactory.ARRAY_SIZE_GETTER);
		sizeGetters.add(SizeGetterFactory.OBJECT_SIZE_GETTER);
		sizeGetter=new ReflectSizeGetter(sizeGetters);
	}
	public HolmosMemoryCacheStore() {
		cacheSize=0;
	}
	
	/**
	 * 缓存数据仓库
	 * <li>初始化大小=CacheConstValue.STORE_INIT_CAPACITY 可以存储10000个元素，如果达到了最大限制，则不能再存入，如果10000个到了，但是没有达到最大值，name按照第二个参数进行扩容</li>
	 * <li>扩容速度，每次容量增加100*CacheConstValue.STORE_LOAD_FACTOR%即 75%</li>
	 * <li>多少个线程可以并行的吸入map=CacheConstValue.STORE_CONCURRENCY_LEVEL=100</li>
	 * */
	private Map<Object,HolmosCacheData> cacheStore=new ConcurrentHashMap<Object, HolmosCacheData>(
			CacheConstValue.STORE_INIT_CAPACITY, CacheConstValue.STORE_LOAD_FACTOR, CacheConstValue.STORE_CONCURRENCY_LEVEL);
	
	@Override
	public HolmosResultCode putData(HolmosCacheData data) {
		data.setCreatedTime(System.currentTimeMillis());
		data.updated();
		if(isLocked(data.getKey())){
			return HolmosResultCode.LOCKED;
		}
		cacheStore.put(data.getKey(), data);
		cacheSize += sizeGetter.getDataSize(data.getValue());
		return HolmosResultCode.SUCCESS;
	}

	@Override
	public HolmosResultCode putBatchData(List<HolmosCacheData> data) {
		boolean allSucceed=true;
		for(HolmosCacheData dataTemp:data){
			dataTemp.updated();
			if(isLocked(dataTemp.getKey()))
				allSucceed=false;
			else{
				cacheStore.put(dataTemp.getKey(), dataTemp);
				cacheSize += sizeGetter.getDataSize(dataTemp.getValue());
			}
		}
		return allSucceed?HolmosResultCode.SUCCESS:HolmosResultCode.PARTSUCC;
	}

	@Override
	public HolmosResultCode removeData(Object key) {
		cacheStore.remove(key);
		return HolmosResultCode.SUCCESS;
	}

	@Override
	public HolmosResultCode removeBatchData(List<? extends Object> keys) {
		for(Object key:keys)
			cacheStore.remove(key);
		return HolmosResultCode.SUCCESS;
	}

	@Override
	public HolmosDataResult<HolmosCacheData> getData(Object key) {
		HolmosCacheData data=cacheStore.get(key);
		if(data!=null){
			data.visited();
			return new HolmosDataResult<HolmosCacheData>(data,HolmosResultCode.SUCCESS);
		}
		return new HolmosDataResult<HolmosCacheData>(null, HolmosResultCode.DATANOTEXSITS);
	}

	@Override
	public HolmosDataResult<List<HolmosCacheData>> getBatchData(
			List<? extends Object> keys) {
		ArrayList<HolmosCacheData> datas=new ArrayList<HolmosCacheData>();
		boolean allSucceed=true;
		for(Object key:keys){
			HolmosCacheData data=cacheStore.get(key);
			if(data!=null){
				data.visited();
				datas.add(data);
			}
			else
				allSucceed=false;
		}
		if(allSucceed)
			return new HolmosDataResult<List<HolmosCacheData>>(datas, HolmosResultCode.SUCCESS);
		return new HolmosDataResult<List<HolmosCacheData>>(datas, HolmosResultCode.PARTSUCC);
	}

	@Override
	public HolmosResultCode lock(Object key) {
		HolmosCacheData data=cacheStore.get(key);
		if(null==data)
			return HolmosResultCode.DATANOTEXSITS;
		if(data.isLocked())
			return HolmosResultCode.LOCK_EXIST;
		data.setLocked(true);
		data.visited();
	    return HolmosResultCode.SUCCESS;
	}

	@Override
	public HolmosResultCode unlock(Object key) {
		HolmosCacheData data=cacheStore.get(key);
		if(null==data)
			return HolmosResultCode.DATANOTEXSITS;
		if(!data.isLocked())
			return HolmosResultCode.LOCK_NOT_EXIST;
		data.setLocked(false);
		data.visited();
		return HolmosResultCode.SUCCESS;
	}

	@Override
	public HolmosResultCode mlock(List<? extends Object> keys) {
		HolmosCacheData data;
		boolean allSucceed=true;
		for(Object key:keys){
			data=cacheStore.get(key);
			if(null==data||data.isLocked()){
				allSucceed=false;continue;
			}
			data.setLocked(true);
			data.visited();
		}
		return allSucceed?HolmosResultCode.SUCCESS:HolmosResultCode.PARTSUCC;
	}

	@Override
	public HolmosResultCode munlock(List<? extends Object> keys) {
		HolmosCacheData data;
		boolean allSucceed=true;
		for(Object key:keys){
			data=cacheStore.get(key);
			if(null==data||!data.isLocked()){
				allSucceed=false;continue;
			}
			data.setLocked(false);
			data.visited();
		}
		return allSucceed?HolmosResultCode.SUCCESS:HolmosResultCode.PARTSUCC;
	}

	@Override
	public HolmosResultCode clear(boolean quick) {
		if(quick)
			cacheStore.clear();
//		else
			//do something here;
		return HolmosResultCode.SUCCESS;
	}

	@Override
	public HolmosResultCode removeExpireData(boolean quick) {
		for(HolmosCacheData data:cacheStore.values()){
			if(data.isExpire())
				cacheStore.remove(data.getKey());
		}
		return HolmosResultCode.SUCCESS;
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	@Override
	public HolmosResultCode addItems(Object key, List<? extends Object> items,
			int offset, int maxCount) {
		if(offset>=items.size()||offset<=0)
			return HolmosResultCode.OUTOFRANGE;
		HolmosCacheData data=cacheStore.get(key);
		int addSize=items.size()-offset;
		if(List.class.isAssignableFrom(data.getValue().getClass())){
			addSize=addSize>maxCount?maxCount:addSize;
			((List)data.getValue()).addAll(items.subList(offset, addSize+offset));
			data.updated();
			return HolmosResultCode.SUCCESS;
		}else
			return HolmosResultCode.TYPENOTMATCH;
	}

	@SuppressWarnings("rawtypes")
	@Override
	public HolmosResultCode removeItems(Object key, int offset, int count) {
		HolmosCacheData data=cacheStore.get(key);
		if(List.class.isAssignableFrom(data.getValue().getClass())){
			List value=(List) data.getValue();
			if(offset>=value.size()||offset<=0)
				return HolmosResultCode.OUTOFRANGE;
			int end=offset+count>value.size()?value.size():offset+count;
			data.setValue(value.subList(offset, end));
			cacheStore.put(key, data);
			data.updated();
			return HolmosResultCode.SUCCESS;
		}else
			return HolmosResultCode.TYPENOTMATCH;
	}

	@SuppressWarnings("rawtypes")
	@Override
	public HolmosDataResult<HolmosCacheData> getAndRemove(Object key,
			int offset, int count) {
		HolmosCacheData data=cacheStore.get(key);
		if(List.class.isAssignableFrom(data.getValue().getClass())){
			List value=(List) data.getValue();
			if(offset>=value.size()||offset<=0)
				return new HolmosDataResult<HolmosCacheData>(null, HolmosResultCode.OUTOFRANGE);
			int end=offset+count>value.size()?value.size():offset+count;
			data.setValue(value.subList(offset, end));
			cacheStore.put(key, data);
			data.updated();
			return new HolmosDataResult<HolmosCacheData>(data, HolmosResultCode.SUCCESS);
		}else
			return new HolmosDataResult<HolmosCacheData>(null, HolmosResultCode.TYPENOTMATCH);
	}

	@Override
	public boolean isLocked(Object key) {
		if(cacheStore.get(key)==null)
			return false;
		return cacheStore.get(key).isLocked();
	}

	@Override
	public HolmosCacheState getCacheState() {
		return cacheState;
	}

	@Override
	public long getStoreElementsCount() {
		return cacheStore.size();
	}

	@Override
	public long getStoreSize() {
		return cacheSize;
	}

	@Override
	public Set<Object> getAllKeys() {
		return cacheStore.keySet();
	}

	@Override
	public ArrayList<HolmosCacheData> getAllDatas() {
		return (ArrayList<HolmosCacheData>) cacheStore.values();
	}

	@Override
	public HolmosResultCode removeDeletedData() {
		for(HolmosCacheData data : cacheStore.values()){
			if(data.isDeleted())
				cacheStore.remove(data.getKey());
		}
		return HolmosResultCode.SUCCESS;
	}
}
